@@ -30,6 +30,15 @@
 
 /*lint -save -e525 -e539 */
 
+#if defined(XYZZY)
+  #error "Temporary macro XYZZY already defined"
+#endif
+
+#if defined(HAVE_ACCEPT_FILTERS)
+  #define XYZZY MUST_RESTART
+#else
+  #define XYZZY NOT_IMPLEMENTED
+#endif
 PARAM(
 	/* name */	accept_filter,
 	/* typ */	bool,
@@ -37,12 +46,13 @@ PARAM(
 	/* max */	NULL,
 	/* default */	"on",
 	/* units */	"bool",
-	/* flags */	MUST_RESTART,
+	/* flags */	XYZZY,
 	/* s-text */
-	"Enable kernel accept-filters (if available in the kernel).",
+	"Enable kernel accept-filters.",
 	/* l-text */	NULL,
 	/* func */	NULL
 )
+#undef XYZZY
 
 PARAM(
 	/* name */	acceptor_sleep_decay,
@@ -126,6 +136,30 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	ban_cutoff,
+	/* typ */	uint,
+	/* min */	"0",
+	/* max */	NULL,
+	/* default */	"0",
+	/* units */	"bans",
+	/* flags */	EXPERIMENTAL,
+	/* s-text */
+	"Expurge long tail content from the cache to keep the number of bans "
+	"below this value. 0 disables.\n"
+	"This is a safety net to avoid bad response times due to bans being "
+	"tested at lookup time. Setting a cutoff trades response time for "
+	"cache efficiency. The recommended value is proportional to "
+	"rate(bans_lurker_tests_tested) / n_objects while the ban lurker is "
+	"working, which is the number of bans the system can sustain. The "
+	"additional latency due to request ban testing is in the order of "
+	"ban_cutoff / rate(bans_lurker_tests_tested). For example, for "
+	"rate(bans_lurker_tests_tested) = 2M/s and a tolerable latency of "
+	"100ms, a good value for ban_cutoff may be 200K.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
 PARAM(
 	/* name */	ban_lurker_age,
 	/* typ */	timeout,
@@ -137,8 +171,10 @@ PARAM(
 	/* s-text */
 	"The ban lurker will ignore bans until they are this old.  "
 	"When a ban is added, the active traffic will be tested against it "
-	"as part of object lookup.  This parameter "
-	"holds the ban-lurker off, until the rush is over.",
+	"as part of object lookup.  Because many applications issue bans in "
+	"bursts, this parameter holds the ban-lurker off until the rush is "
+	"over.\n"
+	"This should be set to the approximate time which a ban-burst takes.",
 	/* l-text */	"",
 	/* func */	NULL
 )
@@ -176,6 +212,21 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	ban_lurker_holdoff,
+	/* typ */	timeout,
+	/* min */	"0",
+	/* max */	NULL,
+	/* default */	"0.010",
+	/* units */	"seconds",
+	/* flags */	EXPERIMENTAL,
+	/* s-text */
+	"How long the ban lurker sleeps when giving way to lookup"
+	" due to lock contention.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
 PARAM(
 	/* name */	first_byte_timeout,
 	/* typ */	timeout,
@@ -289,6 +340,21 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	clock_step,
+	/* typ */	timeout,
+	/* min */	"0.000",
+	/* max */	NULL,
+	/* default */	"1.000",
+	/* units */	"seconds",
+	/* flags */	0,
+	/* s-text */
+	"How much observed clock step we are willing to accept before "
+	"we panic.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
 PARAM(
 	/* name */	connect_timeout,
 	/* typ */	timeout,
@@ -322,6 +388,8 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_bits.c*/
+/* see tbl/debug_bits.h */
 PARAM(
 	/* name */	debug,
 	/* typ */	debug,
@@ -400,6 +468,8 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_bits.c*/
+/* See tbl/feature_bits.h */
 PARAM(
 	/* name */	feature,
 	/* typ */	feature,
@@ -736,6 +806,7 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	pcre_match_limit,
 	/* typ */	uint,
@@ -751,6 +822,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	pcre_match_limit_recursion,
 	/* typ */	uint,
@@ -799,6 +871,7 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	pool_req,
 	/* typ */	poolparam,
@@ -814,6 +887,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	pool_sess,
 	/* typ */	poolparam,
@@ -829,6 +903,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	pool_vbo,
 	/* typ */	poolparam,
@@ -894,23 +969,6 @@ PARAM(
 	/* func */	NULL
 )
 
-PARAM(
-	/* name */	session_max,
-	/* typ */	uint,
-	/* min */	"1000",
-	/* max */	NULL,
-	/* default */	"100000",
-	/* units */	"sessions",
-	/* flags */	0,
-	/* s-text */
-	"Maximum number of sessions we will allocate from one pool before "
-	"just dropping connections.\n"
-	"This is mostly an anti-DoS measure, and setting it plenty high "
-	"should not hurt, as long as you have the memory for it.",
-	/* l-text */	"",
-	/* func */	NULL
-)
-
 PARAM(
 	/* name */	shm_reclen,
 	/* typ */	vsl_reclen,
@@ -969,15 +1027,39 @@ PARAM(
 	/* func */	NULL
 )
 
-#if 0
+#if defined(HAVE_TCP_FASTOPEN)
+  #define XYZZY MUST_RESTART
+#else
+  #define XYZZY NOT_IMPLEMENTED
+#endif
+PARAM(
+	/* name */	tcp_fastopen,
+	/* typ */	bool,
+	/* min */	NULL,
+	/* max */	NULL,
+	/* default */	"off",
+	/* units */	"bool",
+	/* flags */	XYZZY,
+	/* s-text */
+	"Enable TCP Fast Open extension.",
+	/* l-text */	NULL,
+	/* func */	NULL
+)
+#undef XYZZY
+
+#if defined(HAVE_TCP_KEEP)
+  #define XYZZY	EXPERIMENTAL
+#else
+  #define XYZZY	NOT_IMPLEMENTED
+#endif
 PARAM(
 	/* name */	tcp_keepalive_intvl,
 	/* typ */	timeout,
-	/* min */	"1.000",
-	/* max */	"100.000",
-	/* default */	"5.000",
+	/* min */	"1",
+	/* max */	"100",
+	/* default */	"",
 	/* units */	"seconds",
-	/* flags */	EXPERIMENTAL,
+	/* flags */	XYZZY,
 	/* s-text */
 	"The number of seconds between TCP keep-alive probes.",
 	/* l-text */	"",
@@ -989,9 +1071,9 @@ PARAM(
 	/* typ */	uint,
 	/* min */	"1",
 	/* max */	"100",
-	/* default */	"5",
+	/* default */	"",
 	/* units */	"probes",
-	/* flags */	EXPERIMENTAL,
+	/* flags */	XYZZY,
 	/* s-text */
 	"The maximum number of TCP keep-alive probes to send before giving "
 	"up and killing the connection if no response is obtained from the "
@@ -1003,18 +1085,21 @@ PARAM(
 PARAM(
 	/* name */	tcp_keepalive_time,
 	/* typ */	timeout,
-	/* min */	"1.000",
-	/* max */	"7200.000",
-	/* default */	"600.000",
+	/* min */	"1",
+	/* max */	"7200",
+	/* default */	"",
 	/* units */	"seconds",
-	/* flags */	EXPERIMENTAL,
+	/* flags */	XYZZY,
 	/* s-text */
 	"The number of seconds a connection needs to be idle before TCP "
 	"begins sending out keep-alive probes.",
 	/* l-text */	"",
 	/* func */	NULL
 )
+#undef XYZZY
 
+#if 0
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_add_delay,
 	/* typ */	timeout,
@@ -1030,11 +1115,12 @@ PARAM(
 	"creating threads.\n"
 	"Set this to a few milliseconds if you see the 'threads_failed' "
 	"counter grow too much.\n"
-	"Setting this too high results in insuffient worker threads.",
+	"Setting this too high results in insufficient worker threads.",
 	/* l-text */	"",
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_destroy_delay,
 	/* typ */	timeout,
@@ -1050,6 +1136,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_fail_delay,
 	/* typ */	timeout,
@@ -1076,6 +1163,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_max,
 	/* typ */	thread_pool_max,
@@ -1094,6 +1182,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_min,
 	/* typ */	thread_pool_min,
@@ -1112,6 +1201,36 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
+PARAM(
+	/* name */	thread_pool_reserve,
+	/* typ */	thread_pool_reserve,
+	/* min */	NULL,
+	/* max */	NULL,
+	/* default */	"0",
+	/* units */	"threads",
+	/* flags */	DELAYED_EFFECT| EXPERIMENTAL,
+	/* s-text */
+	"The number of worker threads reserved for vital tasks "
+	"in each pool.\n"
+	"\n"
+	"Tasks may require other tasks to complete (for example, "
+	"client requests may require backend requests). This reserve "
+	"is to ensure that such tasks still get to run even under high "
+	"load.\n"
+	"\n"
+	"Increasing the reserve may help setups with a high number of "
+	"backend requests at the expense of client performance. "
+	"Setting it too high will waste resources by keeping threads "
+	"unused.\n"
+	"\n"
+	"Default is 0 to auto-tune (currently 5% of thread_pool_min).\n"
+	"Minimum is 1 otherwise, maximum is 95% of thread_pool_min.",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_stack,
 	/* typ */	bytes,
@@ -1128,6 +1247,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pool_timeout,
 	/* typ */	timeout,
@@ -1145,6 +1265,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_pools,
 	/* typ */	uint,
@@ -1156,10 +1277,14 @@ PARAM(
 	/* s-text */
 	"Number of worker thread pools.\n"
 	"\n"
-	"Increasing number of worker pools decreases lock contention.\n"
+	"Increasing the number of worker pools decreases lock "
+	"contention. Each worker pool also has a thread accepting "
+	"new connections, so for very high rates of incoming new "
+	"connections on systems with many cores, increasing the "
+	"worker pools may be required.\n"
 	"\n"
 	"Too many pools waste CPU and RAM resources, and more than one "
-	"pool for each CPU is probably detrimal to performance.\n"
+	"pool for each CPU is most likely detrimental to performance.\n"
 	"\n"
 	"Can be increased on the fly, but decreases require a restart to "
 	"take effect.",
@@ -1167,6 +1292,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_queue_limit,
 	/* typ */	uint,
@@ -1176,7 +1302,7 @@ PARAM(
 	/* units */	NULL,
 	/* flags */	EXPERIMENTAL,
 	/* s-text */
-	"Permitted queue length per thread-pool.\n"
+	"Permitted request queue length per thread-pool.\n"
 	"\n"
 	"This sets the number of requests we will queue, waiting for an "
 	"available thread.  Above this limit sessions will be dropped "
@@ -1185,6 +1311,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_pool.c */
 PARAM(
 	/* name */	thread_stats_rate,
 	/* typ */	uint,
@@ -1242,6 +1369,7 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	vcc_allow_inline_c,
 	/* typ */	bool,
@@ -1256,6 +1384,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	vcc_err_unref,
 	/* typ */	bool,
@@ -1270,6 +1399,7 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	vcc_unsafe_path,
 	/* typ */	bool,
@@ -1301,13 +1431,29 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	vsm_free_cooldown,
+	/* typ */	timeout,
+	/* min */	"10.000",
+	/* max */	"600.000",
+	/* default */	"60.000",
+	/* units */	"seconds",
+	/* flags */	0,
+	/* s-text */
+	"How long VSM memory is kept warm after a deallocation "
+	"(granularity approximately 2 seconds).",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
 #if 0
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	vcl_dir,
 	/* typ */	string,
 	/* min */	NULL,
 	/* max */	NULL,
-	/* default */	/opt/varnish/etc/varnish,
+	/* default */	"/opt/varnish/etc/varnish",
 	/* units */	NULL,
 	/* flags */	0,
 	/* s-text */
@@ -1317,16 +1463,17 @@ PARAM(
 	/* func */	NULL
 )
 
+/* actual location mgt_param_tbl.c */
 PARAM(
 	/* name */	vmod_dir,
 	/* typ */	string,
 	/* min */	NULL,
 	/* max */	NULL,
-	/* default */	/opt/varnish/lib/varnish/vmods,
+	/* default */	"/opt/varnish/lib/varnish/vmods",
 	/* units */	NULL,
 	/* flags */	0,
 	/* s-text */
-	"Directory where VCL modules are to be found.",
+	"Directory where Varnish modules are to be found.",
 	/* l-text */	"",
 	/* func */	NULL
 )
@@ -1343,6 +1490,8 @@ PARAM(
 	/* s-text */
 	"Bytes of (req-/backend-)workspace dedicated to buffering VSL "
 	"records.\n"
+	"When this parameter is adjusted, most likely workspace_client "
+	"and workspace_backend will have to be adjusted by the same amount.\n\n"
 	"Setting this too high costs memory, setting it too low will cause "
 	"more VSL flushes and likely increase lock-contention on the VSL "
 	"mutex.\n\n"
@@ -1352,6 +1501,7 @@ PARAM(
 )
 
 #if 0
+/* actual location mgt_param_bits.c*/
 PARAM(
 	/* name */	vsl_mask,
 	/* typ */	vsl_mask,
@@ -1390,7 +1540,7 @@ PARAM(
 	/* name */	vsl_space,
 	/* typ */	bytes,
 	/* min */	"1M",
-	/* max */	NULL,
+	/* max */	"4G",
 	/* default */	"80M",
 	/* units */	"bytes",
 	/* flags */	MUST_RESTART,
@@ -1407,7 +1557,7 @@ PARAM(
 	/* name */	vsm_space,
 	/* typ */	bytes,
 	/* min */	"1M",
-	/* max */	NULL,
+	/* max */	"4G",
 	/* default */	"1M",
 	/* units */	"bytes",
 	/* flags */	MUST_RESTART,
@@ -1420,12 +1570,13 @@ PARAM(
 )
 
 #if 0
+/* see mgt_waiter.c */
 PARAM(
 	/* name */	waiter,
 	/* typ */	waiter,
 	/* min */	NULL,
 	/* max */	NULL,
-	/* default */	kqueue (possible values: kqueue, poll),
+	/* default */	"kqueue (possible values: kqueue, poll)",
 	/* units */	NULL,
 	/* flags */	MUST_RESTART| WIZARD,
 	/* s-text */
@@ -1459,8 +1610,12 @@ PARAM(
 	/* units */	"bytes",
 	/* flags */	DELAYED_EFFECT,
 	/* s-text */
-	"Bytes of HTTP protocol workspace for clients HTTP req/resp.  If "
-	"larger than 4k, use a multiple of 4k for VM efficiency.",
+	"Bytes of HTTP protocol workspace for clients HTTP req/resp.  Use a "
+	"multiple of 4k for VM efficiency.\n"
+	"For HTTP/2 compliance this must be at least 20k, in order to "
+	"receive fullsize (=16k) frames from the client.   That usually "
+	"happens only in POST/PUT bodies.  For other traffic-patterns "
+	"smaller values work just fine.",
 	/* l-text */	"",
 	/* func */	NULL
 )
@@ -1500,4 +1655,38 @@ PARAM(
 	/* func */	NULL
 )
 
+PARAM(
+	/* name */	h2_rx_window_low_water,
+	/* typ */	bytes_u,
+	/* min */	"65535",
+	/* max */	"1G",
+	/* default */	"10M",
+	/* units */	"bytes",
+	/* flags */	WIZARD,
+	/* s-text */
+	"HTTP2 Receive Window low water mark.\n"
+	"We try to keep the window at least this big\n"
+	"Only affects incoming request bodies (ie: POST, PUT etc.)",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
+PARAM(
+	/* name */	h2_rx_window_increment,
+	/* typ */	bytes_u,
+	/* min */	"1M",
+	/* max */	"1G",
+	/* default */	"1M",
+	/* units */	"bytes",
+	/* flags */	WIZARD,
+	/* s-text */
+	"HTTP2 Receive Window Increments.\n"
+	"How big credits we send in WINDOW_UPDATE frames\n"
+	"Only affects incoming request bodies (ie: POST, PUT etc.)",
+	/* l-text */	"",
+	/* func */	NULL
+)
+
+#undef PARAM
+
 /*lint -restore */
